@@ -7,7 +7,6 @@ import logging
 from datetime import timedelta, datetime, date
 
 import elasticsearch
-#from curator import *
 import curator
 
 try:
@@ -19,7 +18,7 @@ except ImportError:
         def emit(self, record):
             pass
 
-__version__ = '2.0.2'
+__version__ = '2.1.0'
 
 # Elasticsearch versions supported
 version_max  = (2, 0, 0)
@@ -119,6 +118,7 @@ def make_parser():
     parser_bloom.set_defaults(func=curator.bloom)
     add_common_args(parser_bloom)
     parser_bloom.add_argument('--older-than', required=True, help='Disable bloom filter cache for indices older than n TIME_UNITs', type=int)
+    parser_bloom.add_argument('--delay', help='Number of seconds to delay after disabling the bloom filter cache of an index.', type=int, default=0)
 
     # Close
     parser_close = subparsers.add_parser('close', help='Close indices')
@@ -142,6 +142,13 @@ def make_parser():
     parser_optimize.add_argument('--max_num_segments', help='Optimize segment count to n segments per shard.', default=DEFAULT_ARGS['max_num_segments'], type=int)
     parser_optimize.add_argument('--delay', help='Number of seconds to delay after optimizing an index.', type=int, default=0)
 
+    # Replicas
+    parser_replicas = subparsers.add_parser('replicas', help='Change replica count of indices')
+    parser_replicas.set_defaults(func=curator.replicas)
+    add_common_args(parser_replicas)
+    parser_replicas.add_argument('--older-than', required=True, help='Change replica count for indices older than n TIME_UNITs', type=int)
+    parser_replicas.add_argument('--replicas', required=True, help='Number of replicas the indices should have', type=int, default=1)
+
     # Show indices
     parser_show = subparsers.add_parser('show', help='Show indices or snapshots')
     parser_show.set_defaults(func=show)
@@ -288,7 +295,7 @@ def main():
             sys.exit(1)
 
     if arguments.command == 'snapshot':
-        if not arguments.older_than and not arguments.most_recent and not arguments.delete_older_than and not arguments.all_indices:
+        if arguments.older_than is None and arguments.most_recent is None and arguments.delete_older_than is None and not arguments.all_indices:
             print('{0} snapshot: error: expect one of --all-indices, --older-than, --most-recent, or --delete-older-than'.format(sys.argv[0]))
             sys.exit(1)
         if arguments.older_than or arguments.most_recent or arguments.all_indices:
